#import "conf.h"
#import "AppControler.h"
#import "Network.h"
#import "Utils.h"

@implementation AppControler
FILE *logFile = 0;

- (int) numberOfRowsInTableView: (NSTableView *)tableView
{
    if ([tableView isEqualTo: hostListTable])
        return [hostList count];
    else if ([tableView isEqualTo: fileListTable])
        return [listName count];
    else
        return 0;
}


- (id) tableView: (NSTableView *)tableView
		objectValueForTableColumn: (NSTableColumn *) tableColumn
		row: (int) row
{
    if ([tableView isEqualTo: hostListTable])
        return [self hostListTableColumn: tableColumn row: row];
    else if ([tableView isEqualTo: fileListTable])
        return [self fileListTableColumn: tableColumn row: row];
    else
        return 0;
}

- (id) hostListTableColumn: (NSTableColumn *)tableColumn
		row: (int)row
{
    return [[hostList objectAtIndex: row] objectForKey: @"title"];
}

- (id) fileListTableColumn: (NSTableColumn *)tableColumn
		row: (int)row
{
//NSString *dirChar[2] = {@"", @"D"};
NSString *identifier = [tableColumn identifier];

    if ([identifier isEqualToString: @"name"])
    {
        if ([detailSwitch state])
            return [listNameLong objectAtIndex: row];
        else
            return [listName objectAtIndex: row];
    }
    else if ([identifier isEqualToString: @"directory"])
    {
        if ([[listDir objectAtIndex: row] intValue])
        {
            //return @"D";
            return [NSImage imageNamed:@"folder.tiff"];
        }
        else
            return 0;
    }
        //return dirChar[[[listDir objectAtIndex: row] intValue]];
    else
        return @"";
}


- (BOOL) tableView:(NSTableView *)tableView writeRows:(NSArray*)rows
	toPasteboard:(NSPasteboard*)pboard
{
	if ([tableView isEqualTo: hostListTable])
        return NO;
	[pboard declareTypes:[NSArray arrayWithObjects:
						//NSFilenamesPboardType, 0] owner:self];	
						//NSFilenamesPboardType, 
						//NSFileContentsPboardType, 
						//NSURLPboardType,	
						//NSTIFFPboardType,
						//@"Apple Uniform Resource Locator",
						//NSStringPboardType,
						0] owner:self];	
	return YES;
}

- (void) pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type
{
	if ([type isEqualToString: NSFilenamesPboardType])
		[sender setPropertyList:[NSArray arrayWithObject: @"/Users/takahasi/Mac.html"]
			forType: NSFilenamesPboardType];
	else if ([type isEqualToString: NSStringPboardType])
		[sender setString: @""
			forType: NSStringPboardType];
}

- (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)local
{
    return NSDragOperationCopy;
}


#if 0 /*dose not work here*/
- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation
{
NSLog(@"dragEnd");
	if (!NSPointInRect(aPoint, [hostWindow frame]))
		[self getFile: self];
}

#endif /*0*/

- (void) awakeFromNib
{
    hostList = [[NSMutableArray alloc] init];
    [hostList addObjectsFromArray: [[NSUserDefaults standardUserDefaults]
            arrayForKey: @"hostList"]];
    if ([hostList count])
    {
        [hostListTable noteNumberOfRowsChanged];
        [hostListTable selectRow: 0 byExtendingSelection: NO];
        [self hostListClick: self];
    }

    [fileListTable setTarget: self];
    [fileListTable setDoubleAction: @selector(fileListDoubleClick:)];
    [hostListTable setTarget: self];
    [hostListTable setDoubleAction: @selector(connect:)];
    
    [fileListDir setDataCell:[[NSImageCell alloc] init]];
    
    [self setUpToolbarToWindow:hostWindow];
    
    [self setMenuEnabled:NO];
    menuEnabled = 0;
}


- (IBAction) connect: (NSButton *) Sender
{
    logFile = fopen([[[logFileField stringValue]
            stringByExpandingTildeInPath] fileSystemRepresentation], "w");
    if (!logFile)
        if (!(logFile = [self newLogFile]))
            return;

    [network setLogFile: logFile];

    [currentLocalPath autorelease];
    currentLocalPath = [localDirectoryField stringValue];
    [currentLocalPath retain];
    
    [network setServerCode: [serverCodePopUp titleOfSelectedItem]];
    [connectWindow close];
    [messageWindow makeKeyAndOrderFront: self];
    
    mlst = YES;
    
    [network connectHost: [hostField stringValue]
        port: 21
        user: [userField stringValue]
        password: [passwordField stringValue]
        path: [pathField stringValue]
        timeout: [timeoutField doubleValue]
        sshTunnel: [sshTunnelSwitch state]
        passive: [passiveSwitch state]
        ssh: [sshSwitch state]
        ];

NSLog(@"connected");
    [hostWindow setTitle: [hostField stringValue]];
    [self refresh];
    [messageWindow close];
    [hostWindow makeKeyAndOrderFront: self];

    menuEnabled = YES;
}


-(FILE *) newLogFile
{
NSSavePanel *savePanel = [NSSavePanel savePanel];
NSString *logFilePath;

    [savePanel setTitle: Localize(@"Save log file as:")];
 
    if ([savePanel runModalForDirectory: NSHomeDirectory()
            file: @"YariTori.log"] != NSOKButton)
        return 0;
    
    logFilePath = [savePanel filename];
    [logFileField setStringValue: logFilePath];
    return fopen([logFilePath fileSystemRepresentation], "w");
}


- (void) refresh
{
    [remoteDirectory autorelease];
    remoteDirectory = [[network pwd] copy];
    [self setHostWindowTitle];
    [listName autorelease];
    [listNameLong autorelease];
    [listDir autorelease];
    listNameLong = [network nameListLongHideDot:[hideDotSwitch state]];
#if 0 /* order in MLSD differs from that in LIST */
    if ([sshSwitch state]
        || [network setNameList: &listName dirList: &listDir
                hideDot:[hideDotSwitch state]])
#endif
    {
        listName = [network nameListHideDot:[hideDotSwitch state]];
        listDir = [self directoryListFromNameLong:listNameLong];
    }
    [listName retain];
    [listNameLong retain];
    [listDir retain];
    [fileListTable reloadData];
}


-(void) setHostWindowTitle
{
NSString *tbs;
NSFont *font = [NSFont titleBarFontOfSize: 0];

    tbs = [NSString stringWithFormat: @"%@:%@",
            [hostField stringValue], remoteDirectory];    
    while ([font widthOfString: tbs] > [hostWindow frame].size.width - 90)
            /* sizeWithAttributes: has a bug */
    {
        tbs = [tbs substringWithRange:NSMakeRange(1, [tbs length]-1)];
    }
    [hostWindow setTitle: tbs];    
}


- (void)windowDidResize:(NSNotification *)aNotification
{
    [self setHostWindowTitle];
}


- (NSArray *) directoryListFromNameLong:(NSArray *)list
{
NSMutableArray *directoryList = [[NSMutableArray alloc] init];
NSEnumerator *en = [list objectEnumerator];
NSString *fileLong;
BOOL isDirectory;
int index = 0;

    for (index = 0; fileLong = [en nextObject]; index++)
    {
        isDirectory = ([fileLong hasPrefix: @"d"]);
        if ([fileLong hasPrefix: @"l"]) /* no 'l' in ssh mode*/
            if (mlst)
                isDirectory = [network mlstIsDirectory: [listName objectAtIndex: index]
                        mlstResult: &mlst];
            else
                isDirectory = YES;
                        
        [directoryList addObject: [NSNumber numberWithInt: isDirectory]];
    }
    return directoryList;
}


- (IBAction) openDirectory: (id) Sender
{
    if ([[listDir objectAtIndex: [fileListTable selectedRow]] intValue])
    {
        [network cd:[listName objectAtIndex: [fileListTable selectedRow]]];
        [self refresh];
    } 
}


- (IBAction) upDirectory: (id) Sender
{
    [network cdup];
    [self refresh];
}


- (IBAction) getFile: (id) Sender
{
NSSavePanel *savePanel = [NSSavePanel savePanel];
NSEnumerator *enumerator = [fileListTable selectedRowEnumerator];
int index = [fileListTable selectedRow];
NSString *remoteFile;
NSString *localFile;
BOOL first = YES;
NSNumber *number;

    if ([fileListTable numberOfSelectedRows] <= 0)
        return;
    //if ([[listDir objectAtIndex: index] intValue])
        //return;
    while (number = [enumerator nextObject])
    {
        index = [number intValue];
        remoteFile = [listName objectAtIndex: index];
        if (first)
        {
            first = NO;
            if ([savePanel runModalForDirectory: currentLocalPath file: remoteFile]
                    != NSOKButton)
                return;
            
            [currentLocalPath autorelease];
            currentLocalPath = [savePanel directory];
            [currentLocalPath retain];
            localFile = [savePanel filename];
        }
        localFile = [currentLocalPath stringByAppendingPathComponent:
                remoteFile];
        if ([[listDir objectAtIndex: index] intValue])
            [self getDirectory: remoteFile to: localFile];
        else
            [network get: remoteFile to: localFile];
    }
}


- (IBAction) fileListDoubleClick: (NSTableView *) Sender
{
    if ([[listDir objectAtIndex: [fileListTable selectedRow]] intValue])
        [self openDirectory: self];
    else
        [self getFile: self];
}


- (IBAction) disconnect: (NSButton *) Sender
{
    [hostWindow close];
}


- (void)windowWillClose:(NSNotification *)aNotification
{
    if (![hostWindow isVisible])
        return;
    [network quit];
    [connectWindow makeKeyAndOrderFront: self];
    if (logFile)
    {
        fclose(logFile);
        logFile = 0;
    }

    menuEnabled = NO;
}


- (void) getDirectory: (NSString *) remoteFile to: (NSString *) localFile
{
NSFileManager *fileManager = [NSFileManager defaultManager];
BOOL exist, isLocalDir;
//NSArray *nameArray;
//NSArray *dirArray;
NSEnumerator *nameEnum;
NSEnumerator *dirEnum;
NSString *currentFile;
NSNumber *currentDir;

    exist = [fileManager fileExistsAtPath: localFile isDirectory: &isLocalDir];
    
    if (exist && !isLocalDir)
        [fileManager removeFileAtPath: localFile handler: nil];

    if ([sshSwitch state])
    {
        [network getSSHDirectory: remoteFile to: localFile];
        return;
    }

    if (!exist || !isLocalDir)
        [fileManager createDirectoryAtPath: localFile attributes: nil];
    [fileManager changeCurrentDirectoryPath: localFile];
    
    [network cd: remoteFile];
    nameEnum = [[network nameList] objectEnumerator];
    dirEnum = [[network directoryList] objectEnumerator];
    
    while ((currentFile = [nameEnum nextObject])
            && (currentDir = [dirEnum nextObject]))
    {
        if ([[currentFile lastPathComponent] isEqualToString: @"."]
                || [[currentFile lastPathComponent] isEqualToString: @".."])
            continue;
        if ([currentDir intValue])
        {
            [self getDirectory: currentFile to: currentFile];
            
        }
        else
            [network get: currentFile to: currentFile];
    }
    [network cdup];
    [fileManager changeCurrentDirectoryPath: @".."];
}


- (IBAction) addHostToList: (id) sender
{
NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];
NSString *title;

    title = [titleField stringValue];
    if ([title length] == 0)
        title = [hostField stringValue];
    
    [dict setObject: title forKey: @"title"];
    [dict setObject: [hostField stringValue] forKey: @"host"];
    [dict setObject: [userField stringValue] forKey: @"user"];
    [dict setObject: [passwordField stringValue] forKey: @"password"];
    [dict setObject: [pathField stringValue] forKey: @"path"];
    [dict setObject: [NSNumber numberWithDouble: [timeoutField doubleValue]]
            forKey: @"timeout"];
    [dict setObject: [logFileField stringValue] forKey: @"logFile"];
    [dict setObject: [localDirectoryField stringValue] forKey: @"localDirectory"];
    [dict setObject: [serverCodePopUp titleOfSelectedItem] forKey: @"serverCode"];
    [dict setObject: [NSNumber numberWithBool: [sshTunnelSwitch state]]
            forKey: @"sshTunnel"];
    [dict setObject: [NSNumber numberWithBool: [crlfSwitch state]]
            forKey: @"crlfConvert"];
    [dict setObject: [NSNumber numberWithBool: [hideDotSwitch state]]
            forKey: @"hideDot"];
    [dict setObject: [NSNumber numberWithBool: [passiveSwitch state]]
            forKey: @"passive"];
    [dict setObject: [NSNumber numberWithBool: [detailSwitch state]]
            forKey: @"detail"];
    [dict setObject: [NSNumber numberWithBool: [sshSwitch state]]
            forKey: @"ssh"];
    
    [hostList addObject: dict];
    [hostListTable reloadData];
	[[NSUserDefaults standardUserDefaults]
           setObject: hostList forKey: @"hostList"];
	[[NSUserDefaults standardUserDefaults] synchronize];
}


- (IBAction) hostListClick: (id) sender
{
unsigned int index = [hostListTable selectedRow];
NSDictionary *dict;

    if (index < 0)
        return;
    dict = [hostList objectAtIndex: index];
    
    [titleField setStringValue: [dict objectForKey: @"title"]];
    [hostField setStringValue: [dict objectForKey: @"host"]];
    [userField setStringValue: [dict objectForKey: @"user"]];
    [passwordField setStringValue: [dict objectForKey: @"password"]];
    [pathField setStringValue: [dict objectForKey: @"path"]];
    [timeoutField setDoubleValue: [[dict objectForKey: @"timeout"] doubleValue]];
    [logFileField setStringValue: [dict objectForKey: @"logFile"]];
    [localDirectoryField setStringValue: [dict objectForKey: @"localDirectory"]];
    [serverCodePopUp selectItemWithTitle: [dict objectForKey: @"serverCode"]];
    [sshTunnelSwitch setState: [[dict objectForKey: @"sshTunnel"] boolValue]];
    [crlfSwitch setState: [[dict objectForKey: @"crlfConvert"] boolValue]];
    [hideDotSwitch setState: [[dict objectForKey: @"hideDot"] boolValue]];
    [self setButtonState: passiveSwitch fromDict: dict forKey: @"passive"
            defaultValue: YES];
    [self setButtonState: detailSwitch fromDict: dict forKey: @"detail"
            defaultValue: NO];
    [self setButtonState: sshSwitch fromDict: dict forKey: @"ssh"
            defaultValue: [[dict objectForKey: @"sshTunnel"] boolValue]];
    if ([sshSwitch state])
        [sshTunnelSwitch setState: NO];
}


-(void) setButtonState:(NSButton *)button fromDict:(NSDictionary *)dict
        forKey:(NSString *)key defaultValue:(BOOL) defaultValue
{
id value;

    value = [dict objectForKey: key];
    if (value)
        [button setState: [value boolValue]];
    else
        [button setState: defaultValue];
}


- (IBAction) removeHostListItem: (NSButton *) sender
{
int index = [hostListTable selectedRow];

    if (index < 0)
        return;

    [hostList removeObjectAtIndex: index];
    [hostListTable deselectAll: self];
    [hostListTable reloadData];
    [[NSUserDefaults standardUserDefaults]
            setObject: hostList forKey: @"hostList"];
}


- (IBAction) setDefaultHost: (NSButton *) sender
{
unsigned int index = [hostListTable selectedRow];

    if (index < 0)
        return;
    
    [hostList insertObject: [hostList objectAtIndex: index] atIndex: 0];
    [hostList removeObjectAtIndex: index + 1];
    [hostListTable reloadData];
    [hostListTable selectRow: 0 byExtendingSelection: NO];
    [[NSUserDefaults standardUserDefaults]
            setObject: hostList forKey: @"hostList"];
}


- (IBAction) putFile: (NSButton *) Sender
{
NSOpenPanel *openPanel = [NSOpenPanel openPanel];
NSString *localFile;
NSEnumerator *enumerator;

    [openPanel setCanChooseDirectories: YES];
    [openPanel setAllowsMultipleSelection: YES];
    [openPanel setResolvesAliases: YES];
    if ([openPanel runModalForDirectory: currentLocalPath file: nil types: nil]
            != NSOKButton)
        return;
    
    [currentLocalPath autorelease];
    currentLocalPath = [openPanel directory];
    [currentLocalPath retain];
    enumerator = [[openPanel filenames] objectEnumerator];
    while (localFile = [enumerator nextObject])
    {
        if ([self isDirectory: localFile])
            [self putDirectory: localFile];
        else
            [network put:localFile crlf:[crlfSwitch state]];
    }
    [self refresh];
}


- (BOOL) isDirectory: (NSString *) path
{
BOOL isDir;

    [[NSFileManager defaultManager] fileExistsAtPath: path isDirectory: &isDir];
    return isDir;
}


- (void) putDirectory: (NSString *) localFile
{
NSFileManager *fileManager = [NSFileManager defaultManager];
NSEnumerator *nameEnum;
NSString *currentFile;
NSString *lastPath;

    if ([sshSwitch state])
    {
        [network putSSHDirectory: localFile];
        return;
    }

    [fileManager changeCurrentDirectoryPath: localFile];
    
    lastPath = [localFile lastPathComponent];
    [network mkdir: lastPath];
    [network cd: lastPath];
    nameEnum = [[fileManager directoryContentsAtPath: localFile] objectEnumerator];
    
    while (currentFile = [nameEnum nextObject])
    {
        if ([self isDirectory: currentFile])
        {
            [self putDirectory: [localFile stringByAppendingPathComponent: currentFile]];
            
        }
        else
            [network put: currentFile crlf:[crlfSwitch state]];
    }
    [network cdup];
    [fileManager changeCurrentDirectoryPath: @".."];
}


- (IBAction) deleteFile: (NSMenuItem *) sender
{
int index = [fileListTable selectedRow];
NSEnumerator *enumerator = [fileListTable selectedRowEnumerator];
NSNumber *number;

    if ([fileListTable numberOfSelectedRows] <= 0)
        return;

    //if (index < 0)
        //return;
    while (number = [enumerator nextObject])
    {
        index = [number intValue];
        if ([[listDir objectAtIndex: index] intValue])
            [self deleteDirectory: [listName objectAtIndex: index]];
        else
            [network delete: [listName objectAtIndex: index]];
    }
    [self refresh];
}


- (void) deleteDirectory: (NSString *) remoteFile
{
NSEnumerator *nameEnum;
NSEnumerator *dirEnum;
NSString *currentFile;
NSNumber *currentDir;

    if ([sshSwitch state])
    {
        [network rmdir: remoteFile];
        return;
    }
    
    [network cd: remoteFile];
    nameEnum = [[network nameList] objectEnumerator];
    dirEnum = [[network directoryList] objectEnumerator];
    while ((currentFile = [nameEnum nextObject])
            && (currentDir = [dirEnum nextObject]))
    {
        if ([[currentFile lastPathComponent] isEqualToString: @"."]
                || [[currentFile lastPathComponent] isEqualToString: @".."])
            continue;
        if ([currentDir intValue])
        {
            [self deleteDirectory: currentFile];
            
        }
        else
            [network delete: currentFile];
    }
    [network cdup];
    
    [network rmdir: remoteFile];
}


- (IBAction) makeDirectory: (NSButton *) sender
{
    [network mkdir: [inputField stringValue]];
    [inputWindow close];
    [self refresh];
}


- (IBAction) coupleButton: (NSButton *) sender
{
    if ((sender == sshTunnelSwitch) && 
            ([sender state] == NSOnState))
    {
        [passiveSwitch setState: YES];
        [sshSwitch setState: NO];
    }
    else if ((sender == passiveSwitch) && 
            ([sender state] == NSOffState))
        [sshTunnelSwitch setState: NO];
    else if ((sender == sshSwitch) && 
            ([sender state] == NSOnState))
    {
        [crlfSwitch setState: NO];
        [sshTunnelSwitch setState: NO];
    }
    else if ((sender == crlfSwitch) && 
            ([sender state] == NSOnState))
    {
        [sshSwitch setState: NO];
    }
}


- (void) setUpToolbarToWindow:(NSWindow *) window
{
NSToolbar *toolbar;

    toolbarDict = [[NSMutableDictionary alloc] init];
    
    [self setToolbarItemName:@"cdup" label:@"UpDirectory"
            tip:@"Move up directory" image:@"up.icns"
            action:@selector(upDirectory:)];

    [self setToolbarItemName:@"openfolder" label:@"Open"
            tip:@"Open Directory" image:@"open.icns"
            action:@selector(openDirectory:)];

    [self setToolbarItemName:@"download" label:@"Download"
            tip:@"Download files" image:@"fileget.icns"
            action:@selector(getFile:)];
    
    [self setToolbarItemName:@"upload" label:@"Upload"
            tip:@"Upload files" image:@"fileput.icns"
            action:@selector(putFile:)];
    
    [self setToolbarItemName:@"closeconnection" label:@"Disconnect"
            tip:@"Disconnect to the server"
            image:@"disconnect.icns"
            action:@selector(disconnect:)];
    
    toolbar = [[[NSToolbar alloc] initWithIdentifier:@"HostToolbar"]
                    autorelease];
    [toolbar setDelegate:self];
    [toolbar setAllowsUserCustomization:NO];
    [toolbar setDisplayMode:NSToolbarDisplayModeIconAndLabel];
    [window setToolbar:toolbar];
}

-(void) setToolbarItemName:(NSString *)name label:(NSString *)label
            tip:(NSString *)tip image:(NSString *)image
            action:(SEL)action
{
NSToolbarItem *item;

    item = [[[NSToolbarItem alloc] initWithItemIdentifier:name]
                autorelease];
    [item setLabel:Localize(label)];
    [item setPaletteLabel:Localize(label)];
    [item setToolTip:Localize(tip)];
    [item setImage:[NSImage imageNamed:image]];
    [item setTarget:self];
    [item setAction:action];
    
    [toolbarDict setObject:item forKey:name];
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar
    itemForItemIdentifier:(NSString *)itemIdentifier
    willBeInsertedIntoToolbar:(BOOL)flag
{
    return [toolbarDict objectForKey:itemIdentifier];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:@"cdup", @"openfolder",
            NSToolbarSpaceItemIdentifier, @"download", @"upload", 
            NSToolbarFlexibleSpaceItemIdentifier, @"closeconnection",
            nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
    return [self toolbarDefaultItemIdentifiers:toolbar];
}



-(IBAction) copyURL:(NSMenu *)sender
{
NSPasteboard *pb = [NSPasteboard generalPasteboard];
NSEnumerator *enumerator = [fileListTable selectedRowEnumerator];
NSString *remoteFile;

    [pb declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:0];
    if ([fileListTable numberOfSelectedRows] <= 0)
        remoteFile = @"";
    else
        remoteFile = [listName objectAtIndex:
                [[enumerator nextObject] intValue]];
    [pb setString: [NSString stringWithFormat:@"ftp://%@%@/%@", 
            [hostField stringValue], remoteDirectory, remoteFile]
            forType: NSStringPboardType];
}


-(BOOL) validateMenuItem:(NSMenuItem *) menuItem
{
    return menuEnabled;
}

-(void) setMenuEnabled:(BOOL)enabled
{
    [deleteMenuItem setEnabled:enabled];
    [mkdirMenuItem setEnabled:enabled];
    [permissionMenuItem setEnabled:enabled];
    [copyURLMenuItem setEnabled:enabled];
}


-(IBAction) willMakeDirectory:(NSMenuItem *)sender
{
    [inputWindow makeKeyAndOrderFront:sender];
}


-(IBAction) willChangePermission: (NSMenuItem *) sender
{
    if ([fileListTable numberOfSelectedRows] <= 0)
        return;
    [self readPermission];
    [permissionWindow makeKeyAndOrderFront:sender];
}


-(void) readPermission
{
int index = [fileListTable selectedRow];
const char *nameLong;

	nameLong = [[listNameLong objectAtIndex: index] cString];
    
    [userReadSwitch setState: nameLong[1] != '-'];
    [userWriteSwitch setState: nameLong[2] != '-'];
    [userExecuteSwitch setState: nameLong[3] != '-'];
    [groupReadSwitch setState: nameLong[4] != '-'];
    [groupWriteSwitch setState: nameLong[5] != '-'];
    [groupExecuteSwitch setState: nameLong[6] != '-'];
    [otherReadSwitch setState: nameLong[7] != '-'];
    [otherWriteSwitch setState: nameLong[8] != '-'];
    [otherExecuteSwitch setState: nameLong[9] != '-'];
}


-(IBAction) setPermission: (NSButton *) sender
{
int permInt;
NSEnumerator *enumerator = [fileListTable selectedRowEnumerator];
NSNumber *number;

    [permissionWindow orderOut: self];
    permInt = 100 * ( 4 * [userReadSwitch state]
            + 2 * [userWriteSwitch state]
            + [userExecuteSwitch state])
            + 10 * ( 4 * [groupReadSwitch state]
            + 2 * [groupWriteSwitch state]
            + [groupExecuteSwitch state])
            + 4 * [otherReadSwitch state]
            + 2 * [otherWriteSwitch state]
            + [otherExecuteSwitch state];
    
    while (number = [enumerator nextObject])
    {
        [network setPermission: permInt toFile:
                [listName objectAtIndex: [number intValue]]];
    }

    [self refresh];
}
@end
