#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>

#include <string.h>

#import "Utils.h"

char omeRootPath[1024] = "";
char omeSettingPath[1024] = "";
NSString *omeRootPathString;

int SetOMEPath(void)
{
char rootFilePath[1024];

	if (omeRootPath[0])
		return 0;

	if (SetOMESettingPath())
		return -1;
	if (!strcpy(rootFilePath, omeSettingPath))
		return -1;
	strcat(rootFilePath, "/OME_Root");
    SFResolveAliasPath(rootFilePath, omeRootPath, 1024);
    omeRootPathString = [[NSString alloc] initWithUTF8String: omeRootPath];
	//strcpy(omeRootPath, C2Path(omeRootPath));

	return 0;
}


int SetOMESettingPath(void)
{
	if (omeSettingPath[0])
		return 0;
	if (!strcpy(omeSettingPath, getenv("HOME")))
		return -1;
	strcat(omeSettingPath, "/Library/Preferences/OME_Preferences/");
        NSString *pathString = [NSString stringWithCString: omeSettingPath];
        if ([[NSFileManager defaultManager] fileExistsAtPath:pathString])
            return 0;
        strcpy(omeSettingPath, getenv("HOME"));
	strcat(omeSettingPath, "/Library/Preferences/OME_Preferences.localized/");
        pathString = [NSString stringWithCString: omeSettingPath];
        if ([[NSFileManager defaultManager] fileExistsAtPath:pathString])  
            return 0;
        return -1;
}


char *C2Path(const char *cString)
{
	return (char *)[[NSString stringWithCString: cString] fileSystemRepresentation];
}


NSString *Path2String(const char *path)
{
	return [[NSFileManager defaultManager]
			stringWithFileSystemRepresentation: path length: strlen(path)];
}


BOOL IsPathDirectory(const char *path)
{
    BOOL isDir = NO;
    BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath: Path2String(path)
					isDirectory:&isDir];
    return (exists && isDir);
}


NSString *Localize(NSString *word)
{
	return 	[[NSBundle mainBundle] localizedStringForKey: word
					value: word table: nil];
}


OSErr SFPath2Spec(const char *fullPath, FSSpec *fssP)
{
FSRef fsr;
Boolean isDirectory;
OSErr error;

	error = FSPathMakeRef(fullPath, &fsr, &isDirectory);
	if (!error)
		error = FSGetCatalogInfo(&fsr, kFSCatInfoNone, 0, 0, fssP, 0);
	return error;
}


OSErr SFSpec2Path(FSSpec *fssP, char *fullPath, unsigned int maxLength)
{
FSRef fsr;
OSErr error;
    error = FSpMakeFSRef(fssP, &fsr);
    if (!error)
        error = FSRefMakePath(&fsr, fullPath, maxLength);
    return error;
}


OSErr SFResolveAliasPath(const char *sourcePath, char *destPath,
        unsigned int maxLength)
{
OSErr error;
FSSpec fss;
Boolean isFolder;
Boolean wasAliased;

    if (error = SFPath2Spec(sourcePath, &fss))
        return error;
	if (error = ResolveAliasFile(&fss, 1, &isFolder, &wasAliased))
        return error;
    error = SFSpec2Path(&fss, destPath, maxLength);
    return error;
}


NSString *OMERelativePath(NSString *path)
{
    if (![path hasPrefix: @"/"])
        path = [omeRootPathString stringByAppendingPathComponent:
                            path];
    return path;
}


int HeadStrCL(const char *keyword, const char *target)
{
int length;
int i;

	length = strlen(keyword);
	for (i = 0; i < length; i++)
	{
		if (tolower(keyword[i]) != tolower(target[i]))
			return 0;
	}
	return 1;
}


void GetPathBase(const char *path, char *base)
{
int i;
int baseStart = 0;

    for (i = 0; path[i]; i++)
    {
        if (path[i] == '/')
            baseStart = i+1;
    }
    
    strcpy(base, path + baseStart);
}