/*
 *  pipe.c
 *  YariTori
 *
 *  Created by Satoshi Takahashi on Fri Nov 21 2003.
 *  Copyright (c) 2003 Satoshi Takahashi. All rights reserved.
 *
 */

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>

/*#include "pipe.h"*/

#define BUFLEN 4096
#define SWAPLEN 3072

struct pipe
{
    int io[2];
    char buf[2][BUFLEN];
    int curBuf;
    int pos;
    int last;
    char prompt[256];
    fd_set readfds;
    struct timeval timeout;
} 
pv =
{
    {-1, -1},
    {"", ""},
    0,
    0,
    0,
    "",
    {{0}},
    {1, 0},
};

int PipeNewChild(const char *path, char *name, ...)
{
int pid;
int argc = 0;
char *argv[100];
va_list ap;

    if (pipe(pv.io) < 0)
        exit(-1);
    
    
fprintf(stderr, "pipe: %d, %d\n", pv.io[0], pv.io[1]);
    
    if ((pid = fork()) < 0)
        exit(-1);
    
    if (pid == 0)
    {
        dup2(pv.io[0], 0);
        dup2(pv.io[1], 1);
        //dup2(pv.io[1], 2);
        
fprintf(stderr, "pipe child\n");
        argv[0] = name;
        va_start(ap, name);
        for (argc=1; argc<99; argc++)
        {
            if ((argv[argc] = va_arg(ap, char *)) == 0)
                break;
        }
        va_end(ap);
        
        execv(path, argv);
        exit(-1);
    }
    
    FD_ZERO(&(pv.readfds));
    FD_SET(pv.io[0], &(pv.readfds));

    return pv.io[1];
}


char *PipeGetS(char *outS, int maxChars)
{
int nRead;
int copyLen;

    while (1)
    {
fprintf(stderr, "PipeGetS 2: %s\n", pv.buf[pv.curBuf]);
fprintf(stderr, "d1\n");
fflush(stderr);
        if (PipeFindNewLine(outS, maxChars))
            return outS; /*NL found or maxChars*/
fprintf(stderr, "d2\n");
fflush(stderr);
        
        if (pv.pos > SWAPLEN)
        {
            pv.last -= pv.pos;
            strncpy(pv.buf[1-pv.curBuf], pv.buf[pv.curBuf] + pv.pos,
                        pv.last);
            pv.pos = 0;
        }
    
fprintf(stderr, "PipeGetS 0: %s\n", pv.buf[pv.curBuf]);
fprintf(stderr, "d3\n");
fflush(stderr);
        nRead = SelectRead(pv.io[0], pv.buf[pv.curBuf] + pv.pos,
                        BUFLEN - pv.pos);
                        //1);
       pv.buf[pv.curBuf][pv.pos + nRead] = 0;
fprintf(stderr, "nRead = %d\n", nRead);
fprintf(stderr, "PipeGetS 1: %s\n", pv.buf[pv.curBuf]);
fflush(stderr);
        if (nRead<=0)
        {
            if (strcmp(pv.buf[pv.curBuf] + pv.pos, pv.prompt) == 0)
                return 0;
            
            if (pv.pos >= pv.last)
                return 0;

            if (maxChars < pv.last - pv.pos)
                copyLen = maxChars;
            else
                copyLen = pv.last - pv.pos;
            strncpy(outS, pv.buf[pv.curBuf] + pv.pos, copyLen);
            pv.pos += copyLen;
            return outS;
        }
        pv.last += nRead;
        pv.buf[pv.curBuf][pv.last] = 0;
fprintf(stderr, "PipeGetS: %s", pv.buf[pv.curBuf]);
     }               

    return 0;
}


int PipeFindNewLine(char *outS, int maxChars)
{
int i;

    for (i = pv.pos; (i < pv.last) && (i < pv.pos + maxChars) ; i++)
    {
        if (pv.buf[pv.curBuf][i] == '\n')
            break;
    }

    if (i < pv.last)
    {
        strncpy(outS, pv.buf[pv.curBuf] + pv.pos,
                    i - pv.pos);
        pv.pos = i;
        if (pv.buf[pv.curBuf][i] == '\n')
           pv.pos++; 
        return 1;
    }
    
    return 0;
}


void PipeSetPrompt(const char * newPrompt)
{
    pv.prompt[255] = 0;
    strncpy(pv.prompt, newPrompt, 255);
}


void PipeSetTimeout(struct timeval *timeout)
{
    pv.timeout = *timeout;
}


int SelectRead(int d, char *buf, int maxByte)
{
int nfds;

    nfds = select(pv.io[0]+1, &(pv.readfds), 0, 0, &(pv.timeout));
    if (nfds)
        return read(d, buf, maxByte);
    return 0;
}

